@@ -11,7 +11,7 @@
 #  See accompanying file LICENSE.txt or copy at
 #      http://www.gnu.org/licenses/gpl-3.0.html
 #
-#  Website: https://www.assembla.com/spaces/pyeasydev/wiki
+#  Website: https://github.com/cokelaer/easydev
 #  Documentation: http://packages.python.org/easydev
 #
 ##############################################################################
@@ -22,31 +22,96 @@
     from easydev.config_tools import *
 
 """
-from ConfigParser import ConfigParser
+try:
+    from ConfigParser import ConfigParser
+except ImportError:
+    from configparser import ConfigParser
+
+
 import os
-from collections import OrderedDict
+import sys
+
+try:
+    from collections import OrderedDict
+except:
+    # python 2.6 and below
+    from ordereddict import OrderedDict
+
+__all__ = ["DynamicConfigParser", "ConfigExample"]
+
+
+
+class _DictSection(object):
+    '''Dictionary section.
+
+    Reference: https://gist.github.com/dangoakachan/3855920
+    This is a temporary solution while I can find a solution
+    for the OrderedDictAttribute
+
+    '''
+
+    def __init__(self, config, section):
+        object.__setattr__(self, '_config', config)
+        object.__setattr__(self, '_section', section)
+
+    def __getattr__(self, attr):
+        return self.get(attr, None)
+    __getitem__ = __getattr__
+
+    def get(self, attr, default = None):
+        if attr in self:
+            return self._config.get(self._section, attr)
+        else:
+            return default
+
+    def __setattr__(self, attr, value):
+        if attr.startswith('_'):
+            object.__setattr__(self, attr, value)
+        else:
+            self.__setitem__(attr, value)
+
+    def __setitem__(self, attr, value):
+        if self._section not in self._config:
+            self._config.add_section(self._section)
+
+        self._config.set(self._section, attr, str(value))
+
+    def __delattr__(self, attr):
+        if attr in self:
+            self._config.remove_option(self._section, attr)
+
+    __delitem__ = __delattr__
+
+    def __contains__(self, attr):
+        config = self._config
+        section = self._section
+
+        return config.has_section(section) and config.has_option(section, attr)
+
+
 
-__all__ = ["OrderedDictAttribute", "DynamicConfigParser", "ConfigExample"]
 
 
 class OrderedDictAttribute(OrderedDict):
-    """This is an extension of :class:`~collections.OrderedDict` class with key as read/write attribute
-    
+    """This is an extension of :class:`~collections.OrderedDict` class
+
+    The added value is the exposure of key as attribute
+
     Consider the following example. The key **test** can be altered
-    as a normal attribute. 
-    
+    as a normal attribute.
+
     .. doctest::
 
-        >>> from easydev.config_tools import OrderedDictAttribute   
-        >>> o = OrderedDictAttribute()    
+        >>> from easydev.config_tools import OrderedDictAttribute
+        >>> o = OrderedDictAttribute()
         >>> o['test'] = 1
         >>> o.test   # equivalent to o['test']
         1
         >>> o.test = 1 # equivalent to o['test'] = 1
-        
+
     .. warning:: the key-attribute can be altered **only** if already
         added in the dictionary itself.
-        
+
     .. doctest::
 
         >>> from easydev.config_tools import OrderedDictAttribute
@@ -55,25 +120,31 @@ class OrderedDictAttribute(OrderedDict):
         >>> o.test     #doctest: +SKIP this is an attribute (not a key)
         1
         >>> o.has_key("test")
-        False 
-        
+        False
+
     .. todo:: when setting an attribute that is not a key, we could
         add it in the dictionary.
-        
+
+    .. warning:: works only for python 2.7 and below.
+
     """
     def __init__(self, *args, **kwds):
         super(OrderedDictAttribute, self).__init__(*args, **kwds)
-        
-        # The spirit of this class is to have a mapping between keys and 
+        if sys.version_info.major < 3:
+            raise Exception("Not available in version 3")
+        else:
+            pass
+
+        # The spirit of this class is to have a mapping between keys and
         # an attribute with the same name. Yet, one can set an attribute
         # while the key hasnot yet been set, which is not expected usage
-        
+
         # FIXME one could solve this issue by creating the key on the fly.
-        
+
         # meanwhile, we will use a print statement to inform the user that this attribute
-        # is not yet a key. 
-        
-        # Yet, when we create an instance of this class, the message will 
+        # is not yet a key.
+
+        # Yet, when we create an instance of this class, the message will
         # be printed. To prevent this, we use th e _init attribute below
         # and catch its presence in the __setattr__ method
         self._init = True  # used in set attributete to figure out if
@@ -88,13 +159,13 @@ class OrderedDictAttribute(OrderedDict):
             return super(OrderedDictAttribute, self).__setitem__(attr, value)
         else:
             if "_init" in self.__dict__.keys():
-                print("warning: attribute {} not in the dictionary yet.".format(attr))
+                print("warning: attribute {0} not in the dictionary yet.".format(attr))
                 print("If you want this attribute to be in the dictionary, you must add it first as a key")
-            
+
             return super(OrderedDictAttribute, self).__setattr__(attr, value)
 
-    
-                     
+
+
 
 class ConfigExample(object):
     """Create a simple example of ConfigParser instance to play with
@@ -136,7 +207,7 @@ class ConfigExample(object):
 
 
 
-class DynamicConfigParser(ConfigParser):
+class DynamicConfigParser(ConfigParser, object):
     """Enhanced version of Config Parser
 
     Provide some aliases to the original ConfigParser class and
@@ -163,55 +234,62 @@ class DynamicConfigParser(ConfigParser):
 
     >>> print c
 
-    .. warning:: if you set options manually (e.G. self.GA.test =1 if GA is a 
+    .. warning:: if you set options manually (e.G. self.GA.test =1 if GA is a
         section and test one of its options), then the save/write does not work
         at the moment even though if you typoe self.GA.test, it has the correct value
-        
+
 
     Methods inhereited from ConfigParser are available:
 
     ::
-    
+
         # set value of an option in a section
         c.set(section, option, value=None)
         # but with this class, you can also use the attribute
         c.section.option = value
-        
+
         # set value of an option in a section
         c.remove_option(section, option)
         c.remove_section(section)
-        
-    
-    """
-    def __init__(self, config_or_filename=None):
-        # why not a super usage here ? Maybe there wee issues related to old style class ?
-        ConfigParser.__init__(self)
-        
-        # I'm hacking the ConfigParser to replace the _sections OrderedDict
-        # by an OrderedDictAttribute to easily access to sections.
-        self._sections = OrderedDictAttribute()
 
-        
-        self._config = None
-        # set the sections and options
-        if type(config_or_filename) == str:
-            self.read(config_or_filename)
+
+    """
+    def __init__(self, config_or_filename=None, *args, **kargs):
+
+        object.__setattr__(self, '_filename', config_or_filename)
+        # why not a super usage here ? Maybe there wee issues related to old style class ?
+        ConfigParser.__init__(self, *args, **kargs)
+
+        if isinstance(self._filename, str) and os.path.isfile(self._filename):
+            self.read(self._filename)
         elif isinstance(config_or_filename, ConfigParser):
-            config = config_or_filename
-            self._replace_config(config)
+            self._replace_config(config_or_filename)
         elif config_or_filename == None:
             pass
         else:
             raise TypeError("config_or_filename must be a valid filename or valid ConfigParser instance")
 
+
+        #self._config = None
+        ## set the sections and options
+        #if type(config_or_filename) == str:
+        #    self.read(config_or_filename)
+        #elif isinstance(config_or_filename, ConfigParser):
+        #    config = config_or_filename
+        #    self._replace_config(config)
+        #elif config_or_filename == None:
+        #    pass
+        #else:
+        #    raise TypeError("config_or_filename must be a valid filename or valid ConfigParser instance")
+
     def read(self, filename):
         """Load a new config from a filename (remove all previous sections)"""
-        if os.path.isfile("test.ini")==False:
-            raise IOError("filename {} not found".format(filename))
-            
+        if os.path.isfile(filename)==False:
+            raise IOError("filename {0} not found".format(filename))
+
         config = ConfigParser()
         config.read(filename)
-        
+
         self._replace_config(config)
 
     def _replace_config(self, config):
@@ -222,7 +300,7 @@ class DynamicConfigParser(ConfigParser):
         """
         for section in self.sections():
             self.remove_section(section)
-            
+
         for section in config.sections():
             self.add_section(section)
             for option in config.options(section):
@@ -231,18 +309,18 @@ class DynamicConfigParser(ConfigParser):
 
     def get_options(self, section):
         """Alias to get all options of a section in a dictionary
-    
+
         One would normally need to extra each option manually::
 
             for option in config.options(section):
                 config.get(section, option, raw=True)#
-                
+
         then, populate a dictionary and finally take care of the types.
-        
-        .. warning:: types may be changed .For instance the string "True" 
+
+        .. warning:: types may be changed .For instance the string "True"
             is interepreted as a True boolean.
 
-        ..  seealso:: internally, this method uses :meth:`section2dict` 
+        ..  seealso:: internally, this method uses :meth:`section2dict`
         """
         return self.section2dict(section)
 
@@ -273,20 +351,23 @@ class DynamicConfigParser(ConfigParser):
         .. note:: a value (string) found to be True, Yes, true, yes is transformed to True
         .. note:: a value (string) found to be False, No, no, false is transformed to False
         .. note:: a value (string) found to be None; none, "" (empty string) is set to None
-        .. note:: an integer, or float is transformed to float
+        .. note:: an integer is cast into an int
         """
         options = {}
         for option in self.options(section):
             data = self.get(section, option, raw=True)
-            if data in ['True', 'Yes', 'true', 'yes']:
+            if data.lower() in ['true', 'yes']:
                 options[option] = True
-            elif data in ['False', 'false', 'no', 'No']:
+            elif data.lower() in ['false', 'no']:
                 options[option] = False
             elif data in ['None', None, 'none', '']:
                 options[option] = None
             else:
                 try: # numbers
-                    options[option] = self.getfloat(section, option)
+                    try:
+                        options[option] = self.getint(section, option)
+                    except:
+                        options[option] = self.getfloat(section, option)
                 except: #string
                     options[option] = self.get(section, option, raw=True)
         return options
@@ -295,43 +376,25 @@ class DynamicConfigParser(ConfigParser):
         """Save all sections/options to a file.
 
         :param str filename: a valid filename
-  
+
         ::
-         
+
             config = ConfigParams('config.ini') #doctest: +SKIP
             config.save('config2.ini') #doctest: +SKIP
-   
+
         """
         try:
             if os.path.exists(filename) == True:
-                print "Warning: over-writing %s " % filename
+                print("Warning: over-writing %s " % filename)
             fp = open(filename,'w')
-        except Exception, e:
-            print e
+        except Exception as err:
+            print(err)
             raise Exception('filename could not be opened')
-     
-          
+
+
         self.write(fp)
         fp.close()
 
-    def add_section(self, section):
-        """add a section in the configuration file
-
-        ::
-
-            >>> c = DynamicConfigParser()
-            >>> c.add_section("general")
-
-        .. note:: this method overloads the parent's method to make sure
-            that OrderedDict instance are AttributeOrdredDict.
-        .. note:: here the section added is an :class:`OrderedDictAttribute`
-
-        """
-        if section in self._sections.keys():
-            raise ValueError("Section {} already in the dictionary".format(section))
-        else:
-            self._sections[section] = OrderedDictAttribute()
-        
     def add_option(self, section, option, value=None):
         """add an option to an existing section (with a value)
 
@@ -342,21 +405,9 @@ class DynamicConfigParser(ConfigParser):
             >>> c.add_option("general", "verbose", True)
         """
         assert section in self.sections(), "unknown section"
-        self.set(section, option, value=value)
-    
-    
-    # no need for those methods anymore
-    #def set(self, section, option, value=None):
-    #    ConfigParser.set(self, section, option, value=value)
-
-    # no need for those methods anymore
-    #def remove_option(self, section, option):
-    #    """Remove an option from a section"""
-    #    ConfigParser.remove_option(self, section, option)
-
-    #def remove_section(self, section):
-    #    """Remove a section"""
-    #    ConfigParser.remove_section(self, section)
+        #TODO I had to cast to str with DictSection
+        # whereas it works with OrderedDictAttribute
+        self.set(section, option, value=str(value))
 
     def __str__(self):
         str_ = ""
@@ -370,14 +421,29 @@ class DynamicConfigParser(ConfigParser):
         return str_
 
     def __getattr__(self, key):
-        if self._sections.has_key(key):
-            return self._sections[key]
+        #print('__getattr__(%s) in DictConfig' % key)
+        return _DictSection(self, key)
+    __getitem__ = __getattr__
+
+    def __setattr__(self, attr, value):
+        if attr.startswith('_') or attr:
+            object.__setattr__(self, attr, value)
         else:
-            return getattr(ConfigParser, key)
-            #.__dict__[key]
-            # does not work probably because old-style class
-            #return super(DynamicConfigParser, self).__getattribute__(key)
-             
+            self.__setitem__(attr, value)
+
+    def __setitem__(self, attr, value):
+        if isinstance(value, dict):
+            section = self[attr]
+            for k, v in value.items():
+                section[k] = v
+        else:
+            raise TypeError('value must be a valid dictionary')
+
+    def __delattr__(self, attr):
+        if attr in self:
+            self.remove_section(attr)
+    def __contains__(self, attr):
+        return self.has_section(attr)
 
     def __eq__(self, data):
         # FIXME if you read file, the string "True" is a string
@@ -396,4 +462,3 @@ class DynamicConfigParser(ConfigParser):
                 except:
                     return False
         return True
-
