@@ -10,13 +10,17 @@
 #  See accompanying file LICENSE.txt or copy at
 #      http://www.gnu.org/licenses/gpl-3.0.html
 #
+#  Website: https://github.com/cokelaer/easydev
 #  website: https://pypi.python.org/pypi/easydev
 #
 ##############################################################################
 """Utilities related to the web
 
 """
-import httplib
+try:
+    import httplib
+except ImportError:
+    import http.client as httplib
 
 
 __all__ = ["isurl", "isurl_reachable"]
@@ -29,7 +33,7 @@ def isurl(url):
     return isurl_reachable(url)
 
 
-def isurl_reachable(url):
+def isurl_reachable(url, timeout=10):
     """Checks if an URL exists or nor
 
     :param str url: the url to look for
@@ -37,16 +41,21 @@ def isurl_reachable(url):
 
 
     """
-    if url.startswith("http://"):
-        url = url.split("http://")[1]
-    request = httplib.HTTPConnection(url)
+    if url.startswith("http://") or url.startswith("https://"):
+        url = url.split("//")[1]
+    request = httplib.HTTPConnection(url, timeout=timeout)
     try:
-        request.request("HEAD", '')
+        request.request("HEAD", '/')
     except:
         return False
     # 302 is a redirection
     # 200 is okay
-    if request.getresponse().status in [200, 302]:
+    try:
+        r1 = request.getresponse()
+    except:
+        return False
+
+    if r1.status in [200, 302]:
         return True
     else:
         return False
